%% Code for Day 9 Power correlations, Granger, and Mutual information
% Get set up: Change path to EEGLab and launch EEGLab before we begin (or use your
% shortcut on the shortcuts bar)

% Make sure you can access your sample data (sampleEEGdata.mat)
load '../sampleEEGdata.mat'; % change directory as needed


%% Power Correlations
% ----------------------------------------------------------

    %% First, a word about Pearson vs Spearman correlations -- Figure 27.1

    four_series = [
      % series 1    series 2    series 3     series 4
        10  8.04    10  9.14    10  7.46     8  6.58;
         8  6.95     8  8.14     8  6.77     8  5.76;
        13  7.58    13  8.76    13 12.74     8  7.71;
         9  8.81     9  8.77     9  7.11     8  8.84;
        11  8.33    11  9.26    11  7.81     8  8.47;
        14  9.96    14  8.10    14  8.84     8  7.04;
         6  7.24     6  6.13     6  6.08     8  5.25;
         4  4.26     4  3.10     4  5.39     8  5.56;
        12 10.84    12  9.13    12  8.15     8  7.91;
         7  4.82     7  7.26     7  6.42     8  6.89;
         5  5.68     5  4.74     5  5.73    19 12.50;
        ];

    % plot and compute correlations
    figure
    for i=1:4
        subplot(2,2,i)
        plot(four_series(:,(i-1)*2+1),four_series(:,(i-1)*2+2),'.')
        lsline
        corr_p = corr(four_series(:,(i-1)*2+1),four_series(:,(i-1)*2+2),'type','p');
        corr_s = corr(four_series(:,(i-1)*2+1),four_series(:,(i-1)*2+2),'type','s');
        title([ 'r_p=' num2str(round(corr_p*1000)/1000) '; r_s=' num2str(round(corr_s*1000)/1000) ])
    end
    
      
    
    %% Power correlations over trials times and sites
    edit powerconn.m

%% ASSIGNMENT   -------------------------------------------
% TF_Power and Power Correlations (over trials, over time, full TF with seed)


    %% EXPLORING: Times, Space
    % Plot voltage data to look for times and chans of interest
        
    % Using the Sample Data, 
        load '../sampleEEGdata.mat'; % change directory as needed
        % Laplacian to minimize volume conduction
        EEG.data = laplacian_perrinX(EEG.data, [EEG.chanlocs.X],[EEG.chanlocs.Y],[EEG.chanlocs.Z]);


        % explore the time and topography to choose a channel to focus on
        pop_topoplot(EEG,1, [50:50:800] ,'EEG sample dataset',[4 4] ,0,'electrodes','on');
        
    %% EXPLORING: Times, Space, and FREQS
    % Plot power tf data to look for times and freqs and chans of interest
        
        % load tf decomposition of Laplacian-transformed version of these data (average over trials) 
        load 'tf_Laplacian_sampleEEGdata.mat';
        
        % plot using tfveiwerx to identify potential times, freqs, and channels of interest
        times2display = [-500 1000];  % edge effects outsside this range
        times2display_idx = dsearchn(tf.times',times2display');  % find the time indices that match
        times2display_idx = times2display_idx(1):times2display_idx(2); % fill in all those inbetween the endpoints
        tfviewerx(tf.times(times2display_idx),tf.frex,tf.power(:,:,times2display_idx),tf.chanlocs);
        
        % make a note of Channels, and the time-freq windows of interest
        % samples here

            channel1 = 'PO7';  % will use as seed for over time and tf space
            timewin1 = [ 125 175 ]; % in ms
            freqwin1 = [ 6 8 ]; % in Hz
            channel2 = 'C4';  % for static window in tf over trials
            timewin2 = [ 425 575 ]; % in ms
            freqwin2 = [ 10 12 ]; % in Hz
 
        
    %% Select channels and time-windows of interest
        
        % convert connectivity parameters to indices
        chan1idx = strcmpi({EEG.chanlocs.labels},channel1);
        time1idx = dsearchn(EEG.times',timewin1');
        chan2idx = strcmpi({EEG.chanlocs.labels},channel2);
        time2idx = dsearchn(EEG.times',timewin2');
        freq1idx = dsearchn(frex',freqwin1');  % if you named your frequency vector "frex"
        freq2idx = dsearchn(frex',freqwin2');
  

    %% Get tf decomposition and plot to find freqs of interest
        
        % frequency parameters for time-frequency decomposition
        min_freq =  2;
        max_freq = 40;
        num_frex = 40;
        frex = logspace(log10(min_freq),log10(max_freq),num_frex);

        % other wavelet parameters
        range_cycles = [ 3 10 ];

        % setup wavelet and convolution parameters
        s = logspace(log10(range_cycles(1)),log10(range_cycles(end)),num_frex) ./ (2*pi*frex);
        wavtime = -2:1/EEG.srate:2;
        half_wave = (length(wavtime)-1)/2;

        % convolution parameters
        nWave = length(wavtime);
        nData = EEG.pnts * EEG.trials;
        nConv = nWave + nData - 1;
        
        % Obtain TF decomposition for Channel 1, and Channel 2 (complex values over time freqs and trials)
        % Note: Do not dB correct at single trial level
        
        % Create 2 x 2 set of subplots
        
        % In first row, Plot mean TF_Power over trials for Ch1 and Ch2
        % Here you may wish to dB correct for display purposes only
        
        % In second row, plot scatter plot of Power Correlations over trials in left plot
        % and a time-series of power correlations over time in right plot
        % For the latter, you'll need to decide on a sliding window size and offset for this.  
        % Both channels use this same sliding window.
        % You will use the time-points in the window to obtain the
        % correlation for the center point in the window (for each trial)
        % and then average the correlation over trials.  
        % BONUS -- use Fisher Z score transformation prior to averaging 
        % and transform back to Z-score units

        % Now in a new plot, plot a TF plot for channel 2 of power correlations (across trials) 
        % using channel 1 (time and freq defined above) as the seed. This is
        % a n-times by n-freqs set of correlations graphed in TF space
        % Place a scalebar, and a title that indicates the seed Channel and
        % TF ROI and the target Channel depicted in the plot

    
 
    
%% Catching up from last time: PCA and ICA
edit PCA_ICA_demo.m

    
%% Granger Prediction
% ----------------------------------------------------------

    %% Figure 28.3 (note: this cell can take a while to run)

    % load sample EEG data
    load '../sampleEEGdata.mat'; % change directory as needed

    % Laplacian transform if you wish
    % EEG.data = laplacian_perrinX(EEG.data, [EEG.chanlocs.X],[EEG.chanlocs.Y],[EEG.chanlocs.Z]);
    
    % define channels for granger prediction
    chan1name = 'f6';
    chan2name = 'f5';
    

    % Granger prediction parameters
    timewin = 100; % in ms
    order   =  20; % in ms

    % temporal down-sample results (but not data!)
    times2save = -200:20:1000; % in ms

    % convert parameters to indices
    timewin_points = round(timewin/(1000/EEG.srate));
    order_points   = round(order/(1000/EEG.srate));

    % find the index of those channels
    chan1 = find(strcmpi(chan1name,{EEG.chanlocs.labels}));
    chan2 = find(strcmpi(chan2name,{EEG.chanlocs.labels}));

    % remove ERP from selected electrodes to improve stationarity
    eegdata = bsxfun(@minus,EEG.data([chan1 chan2],:,:),mean(EEG.data([chan1 chan2],:,:),3));


    % convert requested times to indices
    times2saveidx = dsearchn(EEG.times',times2save');

    % initialize
    [x2y,y2x] = deal(zeros(1,length(times2save))); % the function deal assigns inputs to all outputs
    bic = zeros(length(times2save),15); % Bayes info criteria (hard-coded to order=15)

    disp(['Calculating Granger Prediction with ' num2str(length(times2save)) ' time windows']);
    for timei=1:length(times2save)
        
        if mod(timei,5) == 0
            fprintf('%s',num2str(timei));
        else
            fprintf('%s','.');
        end

        % data from all trials in this time window
        tempdata = squeeze(eegdata(:,times2saveidx(timei)-floor(timewin_points/2):times2saveidx(timei)+floor(timewin_points/2)-mod(timewin_points+1,2),:));

        % detrend and zscore all data
        for triali=1:size(tempdata)
            tempdata(1,:,triali) = zscore(detrend(squeeze(tempdata(1,:,triali))));
            tempdata(2,:,triali) = zscore(detrend(squeeze(tempdata(2,:,triali))));

            % At this point with real data, you should check for stationarity
            % and possibly discard or mark data epochs that are extreme stationary violations.
        end

        % reshape tempdata for armorf
        tempdata = reshape(tempdata,2,timewin_points*EEG.trials);

        % fit AR models (model estimation from bsmart toolbox)
        [Ax,Ex] = armorf(tempdata(1,:),EEG.trials,timewin_points,order_points);
        [Ay,Ey] = armorf(tempdata(2,:),EEG.trials,timewin_points,order_points);
        [Axy,E] = armorf(tempdata     ,EEG.trials,timewin_points,order_points);

        % time-domain causal estimate (Equation 28.4)
        y2x(timei)=log(Ex/E(1,1));
        x2y(timei)=log(Ey/E(2,2));

        % test BIC for optimal model order at each time point
        % (this code is used for the following cell)
        for bici=1:size(bic,2)
            % run model
            [Axy,E] = armorf(tempdata,EEG.trials,timewin_points,bici);
            % compute Bayes Information Criteria
            bic(timei,bici) = log(det(E)) + (log(length(tempdata))*bici*2^2)/length(tempdata);
        end
    end
    fprintf('\n');

    % draw lines
    figure
    subplot(211)
    plot(times2save,x2y)
    hold on
    plot(times2save,y2x,'r')
    legend({[ 'GP: ' chan1name ' -> ' chan2name ];[ 'GP: ' chan2name ' -> ' chan1name ]})
    title([ 'Window length: ' num2str(timewin) ' ms, order: ' num2str(order) ' ms' ])
    xlabel('Time (ms)')
    ylabel('Granger prediction estimate')

    subplot(212)
    plot((1:size(bic,2))*(1000/EEG.srate),mean(bic,1),'--.')
    xlabel('Order (converted to ms)')
    ylabel('Mean BIC over all time points')

    [bestbicVal,bestbicIdx]=min(mean(bic,1));
    
    hold on
    plot(bestbicIdx*(1000/EEG.srate),bestbicVal,'mo','markersize',15)

    title([ 'Optimal order is ' num2str(bestbicIdx) ' (' num2str(bestbicIdx*(1000/EEG.srate)) ' ms)' ])
    
    %% Assignment
    
    % Explore Granger prediction for closely-spaced vs distant sites
       
%% Mutual Information
% ----------------------------------------------------------

    %% Mutual Information Figure 29.11
        % load sample data
        load '../sampleEEGdata.mat'; % change directory as needed

        % Laplacian transform if you wish
        % EEG.data = laplacian_perrinX(EEG.data, [EEG.chanlocs.X],[EEG.chanlocs.Y],[EEG.chanlocs.Z]);
    
       % define electrodes and time windows
        electrodes4mi = {'fz';'o1'};
        timewindow = 400; % width in ms 
        times2save = -400:100:1200;

        % convert ms to indices
        timewindowidx = round(timewindow/(1000/EEG.srate)/2);
        times2saveidx = zeros(size(times2save));
        for i=1:length(times2save)
            [junk,times2saveidx(i)]=min(abs(EEG.times-times2save(i)));  % could use dsearchn
        end

        electrodesidx(1) = find(strcmpi(electrodes4mi{1},{EEG.chanlocs.labels}));
        electrodesidx(2) = find(strcmpi(electrodes4mi{2},{EEG.chanlocs.labels}));

        frex = logspace(log10(4),log10(40),20);
        baselinetime = [-500 -200];


        % baseline from ms to idx
        [junk,baseidx(1)] = min(abs(times2save-baselinetime(1)));
        [junk,baseidx(2)] = min(abs(times2save-baselinetime(2)));

        % specify convolution and wavelet info
        time = -1:1/EEG.srate:1;
        half_of_wavelet_size = (length(time)-1)/2;
        n_wavelet     = length(time);
        n_data        = EEG.pnts*EEG.trials;
        n_convolution = n_wavelet+n_data-1;
        % FFT of data
        fft_EEG1 = fft(reshape(EEG.data(electrodesidx(1),:,:),1,EEG.pnts*EEG.trials),n_convolution);
        fft_EEG2 = fft(reshape(EEG.data(electrodesidx(2),:,:),1,EEG.pnts*EEG.trials),n_convolution);


        % initialize outputs
        mi   = zeros(2,length(frex),length(times2save));
        ispc = zeros(length(frex),length(times2save));
        powc = zeros(length(frex),length(times2save));

        for fi=1:length(frex)
            % create wavelet and get its FFT
            fft_wavelet = fft(exp(2*1i*pi*frex(fi).*time) .* exp(-time.^2./(2*(4/(2*pi*frex(fi)))^2)),n_convolution);

            % convolution of each electrode with wavelet
            convres     = ifft(fft_wavelet.*fft_EEG1,n_convolution);
            analytic1   = reshape(convres(half_of_wavelet_size+1:end-half_of_wavelet_size),EEG.pnts,EEG.trials);
            convres     = ifft(fft_wavelet.*fft_EEG2,n_convolution);
            analytic2   = reshape(convres(half_of_wavelet_size+1:end-half_of_wavelet_size),EEG.pnts,EEG.trials);

            for timei = 1:length(times2save)
                datax = analytic1(times2saveidx(timei)-timewindowidx:times2saveidx(timei)+timewindowidx,:);
                datay = analytic2(times2saveidx(timei)-timewindowidx:times2saveidx(timei)+timewindowidx,:);

                % compute MI
                mi(1,fi,timei) = mutualinformationx(log10(abs(datax).^2),log10(abs(datay).^2),50);  % power
                mi(2,fi,timei) = mutualinformationx(angle(datax),angle(datay),20);                  % phase

                % also compute ISPC-time for comparison
                ispc(fi,timei) = mean(abs(mean(exp(1i*(angle(datay)-angle(datax))),2)),1);

                % also compute power correlations-time
                dataxr = tiedrank(abs(datax));
                datayr = tiedrank(abs(datay));
                n = timewindowidx*2+1;
                powc(fi,timei) = mean( 1-6*sum((dataxr-datayr).^2)/(n*(n^2-1)) );  % formula for Spearman (could use corr with spearman option)
            end

            disp([ 'Finished frequency ' num2str(fi) ' out of ' num2str(length(frex)) ]);
        end


        figure, set(gcf,'name',[ 'Mutual information between ' electrodes4mi{1} ' and ' electrodes4mi{2} ])

        for i=1:2
            % plot baseline-subtracted corrected MI
            subplot(2,2,i)
            contourf(times2save,frex,squeeze(mi(i,:,:))-repmat(mean(mi(i,:,baseidx(1):baseidx(2)),3)',1,length(times2save)),40,'linecolor','none')
            set(gca,'clim',[-.075 .075],'yscale','log','ytick',round(logspace(log10(frex(1)),log10(frex(end)),6)))
        end
        subplot(221);
        title('Mutual information based on power');
        colorbar
        subplot(222);
        title('Mutual information based on phase');
        colorbar

        % plot power correlations
        subplot(223)
        contourf(times2save,frex,powc-repmat(mean(powc(:,baseidx(1):baseidx(2)),2),1,length(times2save)),40,'linecolor','none')
        set(gca,'clim',[-.2 .2],'yscale','log','ytick',round(logspace(log10(frex(1)),log10(frex(end)),6)))
        title('Cross trial power correlations');
        colorbar

        % plot phase synchronization
        subplot(224)
        contourf(times2save,frex,ispc-repmat(mean(ispc(:,baseidx(1):baseidx(2)),2),1,length(times2save)),40,'linecolor','none')
        set(gca,'clim',[-.1 .1],'yscale','log','ytick',round(logspace(log10(frex(1)),log10(frex(end)),6)))
        title('ISPC Trials');
        colorbar

        %% Assignment: Mutual Information 
        % For the two sites chosen in the first exercise 
        % Derive the power Correlations using electrode 1 as a seed and
        % full TF on second elctrode
        % Compare this to the MI based on TF Power
        
