%% ASSIGNMENT   -------------------------------------------
% TF_Power and Power Correlations (over trials, over time, full TF with seed)


%% EXPLORING: Times, Space
% Plot voltage data to look for times and chans of interest

% Using the Sample Data,
load '../sampleEEGdata.mat'; % change directory as needed
% Laplacian to minimize volume conduction
EEG.data = laplacian_perrinX(EEG.data, [EEG.chanlocs.X],[EEG.chanlocs.Y],[EEG.chanlocs.Z]);


% explore the time and topography to choose a channel to focus on
pop_topoplot(EEG,1, [50:50:800] ,'EEG sample dataset',[4 4] ,0,'electrodes','on');

%% EXPLORING: Times, Space, and FREQS
% Plot power tf data to look for times and freqs and chans of interest

% load tf decomposition of Laplacian-transformed version of these data (average over trials)
load 'tf_Laplacian_sampleEEGdata.mat';

% plot using tfveiwerx to identify potential times, freqs, and channels of interest
times2display = [-500 1000];  % edge effects outsside this range
times2display_idx = dsearchn(tf.times',times2display');  % find the time indices that match
times2display_idx = times2display_idx(1):times2display_idx(2); % fill in all those inbetween the endpoints
tfviewerx(tf.times(times2display_idx),tf.frex,tf.power(:,:,times2display_idx),tf.chanlocs);

% make a note of Channels, and the time-freq windows of interest
% samples here

% channel1 = 'PO7';  % will use as seed for over time and tf space
% timewin1 = [ 125 175 ]; % in ms
% freqwin1 = [ 6 8 ]; % in Hz
% channel2 = 'C4';  % for static window in tf over trials
% timewin2 = [ 425 575 ]; % in ms
% freqwin2 = [ 10 12 ]; % in Hz

channel1 = 'POz';  % will use as seed for over time and tf space
timewin1 = [ 125 175 ]; % in ms
freqwin1 = [ 6 8 ]; % in Hz
channel2 = 'C3';  % for static window in tf over trials
timewin2 = [ 425 575 ]; % in ms
freqwin2 = [ 10 12 ]; % in Hz


%% Select channels and time-windows of interest

% convert connectivity parameters to indices
chan1idx = strcmpi({EEG.chanlocs.labels},channel1);
time1idx = dsearchn(EEG.times',timewin1');
chan2idx = strcmpi({EEG.chanlocs.labels},channel2);
time2idx = dsearchn(EEG.times',timewin2');


%% Get tf decomposition and plot to find freqs of interest

% frequency parameters for time-frequency decomposition
min_freq =  2;
max_freq = 40;
num_frex = 40;
frex = logspace(log10(min_freq),log10(max_freq),num_frex);

% convert connectivity frequency parameters to indices
freq1idx = dsearchn(frex',freqwin1');  % if you named your frequency vector "frex"
freq2idx = dsearchn(frex',freqwin2');


% other wavelet parameters
range_cycles = [ 3 10 ];

% setup wavelet and convolution parameters
s = logspace(log10(range_cycles(1)),log10(range_cycles(end)),num_frex) ./ (2*pi*frex);
wavtime = -2:1/EEG.srate:2;
half_wave = (length(wavtime)-1)/2;

% convolution parameters
nWave = length(wavtime);
nData = EEG.pnts * EEG.trials;
nConv = nWave + nData - 1;

% Obtain TF decomposition for Channel 1, and Channel 2 (complex values over time freqs and trials)
% Note: Do not dB correct at single trial level

% Create wavelet family (in frequency domain)
wavelet_familyX = nan(length(frex),nConv);

for fi=1:length(frex)
     wavelet  = exp(2*1i*pi*frex(fi).*wavtime) .* exp(-wavtime.^2./(2*s(fi)^2));
     wavelet_familyX(fi,:) = fft(wavelet,nConv);
     wavelet_familyX(fi,:) = wavelet_familyX(fi,:) ./ max(wavelet_familyX(fi,:));
end

% pre-define tfall
tfall = nan(2,length(frex),EEG.pnts,EEG.trials);

% now loop on channels
for chani=1:2

    % compute the FFT of all trials concatenated
    if chani==1
        alldata = reshape( EEG.data(chan1idx,:,:) ,1,[]);
    else
        alldata = reshape( EEG.data(chan2idx,:,:) ,1,[]);
    end

    % Fourier spectrum of data
    dataX = fft(alldata ,nConv );

    % loop over frequencies
    for fi=1:length(frex)

        % now run convolution in one step
        as = ifft(wavelet_familyX(fi,:) .* dataX);
        as = as(half_wave+1:end-half_wave);

        % and reshape back to time X trials
        as = reshape(as, EEG.pnts, EEG.trials );

        % compute power and save for all trials (don't take mean over trials)
        tfall(chani,fi,:,:) = abs(as).^2;
    end
end

% Create 2 x 2 set of subplots
figure('Name','Power Correlations over Trials and Time');

% In first row, Plot mean TF_Power over trials for Ch1 and Ch2
% Here you may wish to dB correct for display purposes only

% Define baseline msec
baseline_window = [-200 0];
% convert baseline time into indices
baseidx = dsearchn(EEG.times',baseline_window')';

% Set plot color range for db power
dblim = [-5 5];

for chani=1:2
    
    mean_tf_power_chan = squeeze(mean(tfall(chani,:,:,:) ,4)); % mean across trials for this channel (gives freqs by time matrix)
    mean_baseline = mean(mean_tf_power_chan(:,baseidx(1):baseidx(2)),2); % for each freq, mean across time interval
    % decibel correction
    mean_tf_power_chan = 10*log10(bsxfun(@rdivide, mean_tf_power_chan, mean_baseline));

    % now plot
    subplot(2,2,chani);
    contourf(EEG.times,frex,mean_tf_power_chan,40,'linecolor','none')
    set(gca,'xlim',[-200 1300],'clim',dblim)
    xlabel('Time (ms)'), ylabel('Frequency (Hz)')
    eval([ 'title([ ''Channel '' channel' num2str(chani) ' ])' ])
end


% In second row, plot scatter plot of Power Correlations over trials in left plot
% and a time-series of power correlations over time in right plot
% For the latter, you'll need to decide on a sliding window size and offset for this.  
% Both channels use this same sliding window.
% You will use the time-points in the window to obtain the
% correlation for the center point in the window (for each trial)
% and then average the correlation over trials.  
% BONUS -- use Fisher Z score transformation prior to averaging 
% and transform back to Z-score units

% Power Correlations over trials
data4corr = nan(2,EEG.trials); % two channels, all trials
% for each channel, take mean acroos the time-freq window (mean across
% freqs and mean accross times) and the squeeze the matrix so it's a vector
% to put into the data4corr matrix
data4corr(1,:) = squeeze(mean(mean(tfall(1,freq1idx(1):freq1idx(2),time1idx(1):time1idx(2),:),2),3));
data4corr(2,:) = squeeze(mean(mean(tfall(2,freq2idx(1):freq2idx(2),time2idx(1):time2idx(2),:),2),3));

subplot(223);
plot(data4corr(1,:),data4corr(2,:),'o')
xlabel([ 'Power: ' channel1 ', ' num2str(freqwin1(1)) '-' num2str(freqwin1(2)) 'Hz, ' num2str(timewin1(1)) '-' num2str(timewin1(2)) 'ms' ])
ylabel([ 'Power: ' channel2 ', ' num2str(freqwin2(1)) '-' num2str(freqwin2(2)) 'Hz, ' num2str(timewin2(1)) '-' num2str(timewin2(2)) 'ms' ])

[r,p] = corr(data4corr','type','spearman');
% multiply by 1000, round, and divide by 1000 -- gives 3 signif digits!
title([ 'Correlation R=' num2str(round(r(1,2)*1000)/1000) ', p=' num2str(round(p(1,2)*1000)/1000)]);


% Now power corr over time -- each channel using same time window but
% unique freqs

% Define sliding window
sliding_window_size = 200; % in msec
sliding_window_offset = 5; % in samples -- could do 1 sample but adjacent correlations will be highly similar
sliding_window_size_pnts = round(sliding_window_size/(1000/EEG.srate));

% pre-define time series for corrs
ts_corr = nan(EEG.pnts,EEG.trials);  % will have nans at beginning and end (time)
                                     % even after we run the code below
                                     % and unless sliding_window_offset is
                                     % 1, there will be many other nans in
                                     % this time dimension

% loop on time indices
start_idx = round(sliding_window_size_pnts/2); % half sliding window size -- center point of window
end_idx = EEG.pnts - round(sliding_window_size_pnts/2); % half sliding window size -- center point of window
corr_time_idx = start_idx:sliding_window_offset:end_idx; 
for time_idx = corr_time_idx
    % create time window centered on current time index
   time_idx_this_window = (time_idx - floor(sliding_window_size_pnts/2)):(time_idx + floor(sliding_window_size_pnts/2));

   % loop on epochs
   for epochi = 1:EEG.trials
       % below we take the mean across frequencies, but retain the time
       % points so this vector is power in the freq band of interest at
       % each timepoint in this window (done for each epoch)
       data_chan_1 = squeeze(mean(tfall(1,freq1idx(1):freq1idx(2),time_idx_this_window,epochi),2));
       data_chan_2 = squeeze(mean(tfall(2,freq2idx(1):freq2idx(2),time_idx_this_window,epochi),2));
       ts_corr(time_idx,epochi) = corr(data_chan_1,data_chan_2,'type','Spearman');
   end
end
mean_ts_corr = mean(ts_corr,2);  % now take mean across epochs

% BONUS do Fisher Z transform before average, and then untransform the
% mean FisherZ
% z_mean_ts_corr = mean(atanh(ts_corr),2);
% un_z_mean_ts_corr = tanh(z_mean_ts_corr);
% THIS SEEMS WONKY giving inf values.  Something wrong, so gave up (for now) on
% fisher z transform.

% now plot
subplot(224);
plot(EEG.times(corr_time_idx),mean_ts_corr(corr_time_idx));
set(gca,'xlim',[-200 1300])
xlabel('msec');
ylabel('Spearman rho');
title(sprintf('%s\n%s', ['Channels ' channel1 ' ' num2str(freqwin1(1)) ' to ' num2str(freqwin1(2)) ' hz'], ...
    ['& ' channel2 ' '  num2str(freqwin2(1)) ' to ' num2str(freqwin2(2))  ' hz using ' num2str(sliding_window_size) ' msec sliding window']));


% Now in a new plot, plot a TF plot for channel 2 of power correlations (across trials)
% using channel 1 (time and freq defined above) as the seed. This is
% a n-times by n-freqs set of correlations graphed in TF space
% Place a scalebar, and a title that indicates the seed Channel and
% TF ROI and the target Channel depicted in the plot

tf_corr = nan(num_frex,EEG.pnts);
% loop on frequencies
for fi = 1:num_frex
    % loop on time
    for ti = 1:EEG.pnts
        tf_corr(fi,ti) = corr(squeeze(tfall(1,fi,ti,:)),squeeze(tfall(2,fi,ti,:)),'type','Spearman');
    end
end

figure('Name',['Correlations with seed ' channel1 ' from ' num2str(timewin1(1)) ' to ' num2str(timewin1(2)) ' msec & ' num2str(freqwin1(1)) ' to ' num2str(freqwin1(2)) ' hz'])
corr_lim = [0 .4];
contourf(EEG.times,frex,tf_corr,40,'linecolor','none')
set(gca,'xlim',[-200 1300],'clim',corr_lim)
xlabel('Time (ms)'), ylabel('Frequency (Hz)')
title(sprintf('%s\n%s', ['Correlations for channel ' channel2 ' with power from ' channel1], [channel1 ' seed is from ' num2str(timewin1(1)) ' to ' num2str(timewin1(2)) ' msec & ' num2str(freqwin1(1)) ' to ' num2str(freqwin1(2)) ' hz']));


