   %% PCA vs ICA with simulated data (code in this cell largely from Lecturelet)
      
        % generate data
        x = [ 1*randn(1000,1) .05*randn(1000,1) ];  % Constrains second dimenstion
        
        % Visualize the data
        figure(1); 
        scatter(x(:,1),x(:,2)); 
        ylim([min(x(:,1)) max(x(:,1))]); % make the x and y axis on same scale

        % rotation matrix  (simply rotates the data in two dimensions)
        th = -pi/6;
        R1 = [ cos(th) -sin(th); sin(th) cos(th) ];
        th = -pi/3;
        R2 = [ cos(th) -sin(th); sin(th) cos(th) ];
        % rotate data
        y = [ x*R1 ; x*R2 ];

        % Plot data (two rotated versions of the same data)
        figure(2), clf
        subplot(221)
        plot(y(:,1),y(:,2),'o')
        datarange = max(y(:))*1.2;
        set(gca,'xlim',[-datarange datarange],'ylim',[-datarange datarange])
        xlabel('X axis'), ylabel('Y axis')
        axis square
        title('Data in XY space')

        %% run PCA
        
        % Zero-mean
        y = bsxfun(@minus,y,mean(y,1));
        % covariance on zero-meaned data
        covmat = (y'*y) / length(y);
        % PCA to get eigen vectors and values
        [evecsY,evalsY] = eig(covmat);
        
        % Plot eigen vectors in XY Space
        hold on
        % A line from origin (1) to scaled legth of eigen vector
        plot(evalsY(1,1)*[0 evecsY(1,1)],evalsY(1,1)*[0 evecsY(2,1)],'r','linew',4);  % smallest PC
        plot(evalsY(2,2)*[0 evecsY(1,2)],evalsY(2,2)*[0 evecsY(2,2)],'r','linew',4);  % largest PC


        % Plot eigen vectors in Component Space
        subplot(222)
        pc1 = y*evecsY(:,1);
        pc2 = y*evecsY(:,2);
        plot(pc2,pc1,'ms')
        datarange = max([pc1(:); pc2(:)])*1.2;
        set(gca,'xlim',[-datarange datarange],'ylim',[-datarange datarange])
        xlabel('PC1 axis'), ylabel('PC2 axis')
        axis square
        title('Data in PC space')


        %% run ICA
        % first plot datapoints in XY spcace
        subplot(223)
        plot(y(:,1),y(:,2),'o')
        datarange = max(y(:))*1.2;
        set(gca,'xlim',[-datarange datarange],'ylim',[-datarange datarange])
        % Get IC vectors
        ivecs = jader(y');
        % Plot in XY Space
        hold on
        plot([0 ivecs(1,1)],[0 ivecs(2,1)],'r','linew',4)
        plot([0 ivecs(1,2)],[0 ivecs(2,2)],'r','linew',4)
        xlabel('X axis'), ylabel('Y axis')
        axis square
        title('Data in XY space')


        % Plot IC vectors in Component Space
        subplot(224)
        ic_scores = ivecs*y';
        plot(ic_scores(1,:),ic_scores(2,:),'ms')
        datarange = max(ic_scores(:))*1.2;
        set(gca,'xlim',[-datarange datarange],'ylim',[-datarange datarange])
        xlabel('IC1 axis'), ylabel('IC2 axis')
        axis square
        title('Data in IC space')

    %% PCA with real data
        % this code adapted from Chapter 23 code from Mike Cohen
        
        % Load sample EEG data 
        load '../Day07/ERN_Data_LM_Ref.mat';  % provided with Day07 download, but change as needed

        % average single-trial covariances
        covar = zeros(EEG.nbchan);
        for i=1:EEG.trials
            eeg = squeeze(EEG.data(:,:,i)) - repmat(squeeze(mean(EEG.data(:,:,i),2)),1,EEG.pnts);
            covar = covar + (eeg*eeg')./(EEG.pnts-1);
        end
        covar = covar./i;

        % principle components analysis via eigenvalue decomposition
        [pc,eigvals] = eig(covar);

        % components are listed in increasing order, and converted here to descending order for convenience
        pc      = pc(:,end:-1:1);
        eigvals = diag(eigvals);  % extracts the diagonal
        eigvals = 100*eigvals(end:-1:1)./sum(eigvals); % convert to percent change and re-order from last to first (biggest now first)

        figure
        for i=1:9 % first 9 PCs -- loading on scalp map
            subplot(3,3,i)
            topoplot(pc(:,i),EEG.chanlocs,'electrodes','off','plotrad',.53);
            title([ 'PC #' num2str(i) ', eigval=' num2str(eigvals(i)) ])
        end
        
        % Plot PC time course
        PC2plot = 1;
        % We're interested in ERP, so take the mean
        ERP = squeeze(mean(EEG.data,3)); % chans by tiempoints
        % it's a weighted sum of channel timecourses
        time_idx = 641:1280; % these data were "elongated" by flipping and adding the real epoch before and after the 640 point actual epoch
        figure; plot(EEG.times(time_idx),pc(:,PC2plot)'*ERP(:,time_idx)); % chans by 1 so transpose before multiplying

    %% PCA on EEG data (ERP vs all trials)

    load '..\sampleEEGdata'
    
    % compute ERP
    erp = squeeze(mean(EEG.data,3));
    
    % subtract mean and compute covariance
    erp      = bsxfun(@minus,erp,mean(erp,2));
    covarERP = (erp*erp')./(EEG.pnts-1);
    
    figure(4), clf
    subplot(121)
    imagesc(covarERP)
    axis square
    set(gca,'xticklabel',{EEG.chanlocs(get(gca,'xtick')).labels},'yticklabel',{EEG.chanlocs(get(gca,'ytick')).labels},'clim',[-1 5])
    title('Covariance of ERP')
    
    
    % average single-trial covariances
    subplot(122)
    covarST = zeros(EEG.nbchan);
    % note that the covariance of each trial is computed separately, then averaged
    for i=1:EEG.trials
        eeg     = bsxfun(@minus,squeeze(EEG.data(:,:,i)),squeeze(mean(EEG.data(:,:,i),2)));
        covarST = covarST + (eeg*eeg')./(EEG.pnts-1);
    end
    covarST = covarST./i;
    imagesc(covarST)
    axis square
    set(gca,'xticklabel',{EEG.chanlocs(get(gca,'xtick')).labels},'yticklabel',{EEG.chanlocs(get(gca,'ytick')).labels},'clim',[20 150])
    title('Average covariance of single-trial EEG')
    
    
    %%
    
    % PCA of ERP
    [evecsERP,evalsERP] = eig(covarERP);
    [evecsST ,evalsST]  = eig(covarST);
    
    figure(5), clf
    subplot(221)
    topoplot(evecsERP(:,end),EEG.chanlocs,'maplimits',[0 .2]);
    title('PCA of ERP, 1^s^t component')
    
    subplot(222)
    topoplot(evecsST(:,end),EEG.chanlocs,'maplimits',[0 .2]);
    title('PCA of EEG, 1^s^t component')
    
    %% now get time courses
    
    % for ERP
    erpV1 = erp'*evecsERP(:,end);
    
    % for EEG
    eeg   = reshape(EEG.data,EEG.nbchan,[]);
    eegV1 = eeg'*evecsST(:,end);
    eegV1 = reshape(eegV1',EEG.pnts,EEG.trials);
    eegV1 = mean(eegV1,2);
    
    subplot(223)
    plot(EEG.times,erpV1,'linew',2)
    set(gca,'xlim',[-200 1200],'ylim',[-50 50])
    
    subplot(224)
    plot(EEG.times,eegV1,'linew',2)
    set(gca,'xlim',[-200 1200],'ylim',[-50 50])

%% PCA vs ICA with real data
        
        % Do this with EEGLab tools
        
        % Load sample EEG data that has ICA results within (saves time vs running it)
        load '../Day07/ERN_Data_LM_Ref.mat';  % provided with Day07 download, but change as needed

        % plot 8 ICs accounting for most variance in the time range -200 to 1000 msec
        pop_topoplot(EEG,0, [1:8] ,'ICAs of EEG sample dataset',[2 4] ,0,'electrodes','on');
        pop_envtopo(EEG, [-200  1000] ,'limcontrib',[-200 1000],'compsplot',[8],'title', 'Largest ICA components of EEG sample dataset','electrodes','off');

        % run PCA using PCA plugin -- overwrites IC fields in EEG
        % PCA_plugin was included if you downloaded EEGLab from the course
        % site many weeks ago
        [tmp1 EEG.icawinv] = runpca(EEG.data(:,:));
        EEG.icaweights = pinv(EEG.icawinv);
        EEG.icasphere = eye(EEG.nbchan);
        clear tmp1;

        % plot 8 PCs accounting for most variance in the time range -200 to 1000 msec
        pop_topoplot(EEG,0, [1:8] ,'PCAs EEG sample dataset',[2 4] ,0,'electrodes','on');
        pop_envtopo(EEG, [-200  1000] ,'limcontrib',[-200 1000],'compsplot',[8],'title', 'Largest PCA components of EEG sample dataset','electrodes','off');

   
